# -*- coding: utf-8 -*-
# vim: set ts=4 et ai sw=4:
#
# plugin.py
#
# Copyright (C) Joan Touzet 2007
# Based on code Copyright (C) Marcos Pinto 2007 <markybob@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, write to:
#     The Free Software Foundation, Inc.,
#     51 Franklin Street, Fifth Floor
#     Boston, MA  02110-1301, USA.
#
#  In addition, as a special exception, the copyright holders give
#  permission to link the code of portions of this program with the OpenSSL
#  library.
#  You must obey the GNU General Public License in all respects for all of
#  the code used other than OpenSSL. If you modify file(s) with this
#  exception, you may extend this exception to your version of the file(s),
#  but you are not obligated to do so. If you do not wish to do so, delete
#  this exception statement from your version. If you delete this exception
#  statement from all source files in the program, then also delete it here.

class plugin_MoveTorrentPlus:
    def __init__(self, path, deluge_core, deluge_interface):
        print "Loading MoveTorrentPlus plugin..."
        #
        # Initialization
        import gtk, os, ConfigParser
        import deluge, deluge.common, deluge.dgtk, deluge.pref
        from deluge import dialogs
        self.timer = 0
        self.debug = False
        self.path = path
        self.core = deluge_core
        self.interface = deluge_interface
        self.window = self.interface.window
        self.dialogs = deluge.dialogs
        self.core.connect_event(self.core.constants['EVENT_STORAGE_MOVED'], self.handle_event)
        self.core.connect_event(self.core.constants['EVENT_FINISHED'], self.handle_event)
        # 
        # Read configuration
        self.config_file = deluge.common.CONFIG_DIR + "/movetorrentplus.conf"
        homedir = os.path.expanduser("~")
        self.config = deluge.pref.Preferences(self.config_file, global_defaults=False, defaults={
                    "default_done_path": homedir,
                    "enable_move_completed": False,
                    "default_requests_path": homedir,
                    "enable_auto_requests": False,
                    "requests_poll_interval": 60,
                    "default_moveto_path":  homedir,
                    "enable_auto_moveto": False,
                    "auto_suffix": ".loaded",
                    "enable_auto_suffix": False
                })
        try: 
            self.config.load()
        except IOError:
            pass
        # Cached for speed in tight update loop
        self.interval = self.config.get("requests_poll_interval")
        #
        # Set up the preferences dialog
        self.glade = gtk.glade.XML(path + "/movetorrentplus.glade")
        self.glade.signal_autoconnect({
                                        'toggle_ui': self.toggle_ui,
                                        'dialog_ok': self.dialog_ok,
                                        'dialog_cancel': self.dialog_cancel
                                      })
        self.dialog = self.glade.get_widget("dialog")
        self.dialog.set_position(gtk.WIN_POS_CENTER)
        self.dialog.set_icon_from_file(self.path + "/movetorrentplus.png")
        #
        # Set up per-torrent pop-up menu
        self.menuitem_image = gtk.Image()
        self.menuitem_image.set_from_stock(gtk.STOCK_SAVE_AS, gtk.ICON_SIZE_MENU)

        self.menuitem = gtk.ImageMenuItem(_("_Move Torrent"))
        self.menuitem.set_image(self.menuitem_image)
        self.menuitem.connect("activate", self.movetorrent_clicked)
        self.interface.torrent_menu.append(self.menuitem)
        self.menuitem.show_all()

    def unload(self):
        self.interface.torrent_menu.remove(self.menuitem)
        self.core.disconnect_event(self.core.constants['EVENT_STORAGE_MOVED'], self.handle_event)
        self.core.disconnect_event(self.core.constants['EVENT_FINISHED'], self.handle_event)
        self.config.save(self.config_file)
    
    def movetorrent_clicked(self, widget):
        unique_ids = self.interface.get_selected_torrent_rows()

        path = self.dialogs.show_directory_chooser_dialog(None, \
                _("Choose a directory to move files to"))
        if path:
            for unique_id in unique_ids:
                self.core.move_storage(unique_id, path)

    def configure(self, window):
        import os.path
        try:
            #
            # Checkboxes
            self.glade.get_widget("chk_move_completed").set_active(self.config.get("enable_move_completed"))
            self.glade.get_widget("chk_auto_requests").set_active(self.config.get("enable_auto_requests"))
            self.glade.get_widget("chk_auto_moveto").set_active(self.config.get("enable_auto_moveto"))
            self.glade.get_widget("chk_auto_suffix").set_active(self.config.get("enable_auto_suffix"))
            #
            # Paths
            self.glade.get_widget("finished_path_button").set_filename(self.config.get("default_done_path"))
            self.glade.get_widget("requests_path_button").set_filename(self.config.get("default_requests_path"))
            self.glade.get_widget("moveto_path_button").set_filename(self.config.get("default_moveto_path"))
            #
            # Spinnaz
            self.glade.get_widget("spinnaz_poll_interval").set_value(float(self.config.get("requests_poll_interval")))
            #
            # Textfield
            self.glade.get_widget("txt_auto_suffix").set_text(self.config.get("auto_suffix"))

            # 
            # Disable auto-request options if not enabled
            self.glade.get_widget("chk_auto_moveto").set_sensitive(
                self.glade.get_widget("chk_auto_requests").get_active())
            self.glade.get_widget("chk_auto_suffix").set_sensitive(
                self.glade.get_widget("chk_auto_requests").get_active())
            self.glade.get_widget("requests_path_button").set_sensitive(
                self.glade.get_widget("chk_auto_requests").get_active())
            self.glade.get_widget("spinnaz_poll_interval").set_sensitive(
                self.glade.get_widget("chk_auto_requests").get_active())
            self.glade.get_widget("moveto_path_button").set_sensitive(
                self.glade.get_widget("chk_auto_requests").get_active() and \
                    self.glade.get_widget("chk_auto_moveto").get_active())
            self.glade.get_widget("txt_auto_suffix").set_sensitive(
                self.glade.get_widget("chk_auto_requests").get_active() and \
                    self.glade.get_widget("chk_auto_suffix").get_active())
            self.glade.get_widget("finished_path_button").set_sensitive(
                self.glade.get_widget("chk_move_completed").get_active())

        except:
            #
            # Checkboxes
            self.glade.get_widget("chk_move_completed").set_active(False)
            self.glade.get_widget("chk_auto_requests").set_active(False)
            self.glade.get_widget("chk_auto_moveto").set_active(False)
            self.glade.get_widget("chk_auto_suffix").set_active(False)
            #
            # Paths
            self.glade.get_widget("finished_path_button").set_sensitive(False)
            self.glade.get_widget("requests_path_button").set_sensitive(False)
            self.glade.get_widget("moveto_path_button").set_sensitive(False)
            #
            # Spinnaz
            self.glade.get_widget("spinnaz_poll_interval").set_sensitive(False)
            #
            # Textfield
            self.glade.get_widget("txt_auto_suffix").set_sensitive(False)

        self.dialog.set_transient_for(window)
        self.dialog.show()

    def toggle_ui(self, widget):
        value = widget.get_active()
        if widget == self.glade.get_widget("chk_auto_requests"):
            self.glade.get_widget("chk_auto_moveto").set_sensitive(value)
            self.glade.get_widget("chk_auto_suffix").set_sensitive(value)
            self.glade.get_widget("spinnaz_poll_interval").set_sensitive(value)
            self.glade.get_widget("requests_path_button").set_sensitive(value)
            self.glade.get_widget("moveto_path_button").set_sensitive(value and \
                self.glade.get_widget("chk_auto_moveto").get_active())
            self.glade.get_widget("txt_auto_suffix").set_sensitive(value and \
                self.glade.get_widget("chk_auto_suffix").get_active())
        elif widget == self.glade.get_widget("chk_auto_moveto"):
            self.glade.get_widget("moveto_path_button").set_sensitive(value)
        elif widget == self.glade.get_widget("chk_auto_suffix"):
            self.glade.get_widget("txt_auto_suffix").set_sensitive(value)
        elif widget == self.glade.get_widget("chk_move_completed"):
            self.glade.get_widget("finished_path_button").set_sensitive(value)
            

    def dialog_ok(self, source):
        self.dialog.hide()
        # 
        # Checkboxes
        self.config.set("enable_move_completed", self.glade.get_widget("chk_move_completed").get_active())
        self.config.set("enable_auto_requests", self.glade.get_widget("chk_auto_requests").get_active())
        self.config.set("enable_auto_moveto", self.glade.get_widget("chk_auto_moveto").get_active())
        self.config.set("enable_auto_suffix", self.glade.get_widget("chk_auto_suffix").get_active())
        #
        # Paths
        self.config.set("default_done_path", self.glade.get_widget("finished_path_button").get_filename())
        self.config.set("default_requests_path", self.glade.get_widget("requests_path_button").get_filename())
        self.config.set("default_moveto_path", self.glade.get_widget("moveto_path_button").get_filename())
        #
        # Spinnaz
        self.config.set("requests_poll_interval", int(self.glade.get_widget("spinnaz_poll_interval").get_value()))
        self.interval = self.config.get("requests_poll_interval")
        #
        # Textfield
        self.config.set("auto_suffix", self.glade.get_widget("txt_auto_suffix").get_text())

    def dialog_cancel(self, source):
        self.dialog.hide()

    def handle_event(self, event):
        if event['event_type'] is self.core.constants['EVENT_STORAGE_MOVED']:
            if event['message'] == self.core.unique_IDs[event['unique_ID']].save_dir:
                self.dialogs.show_popup_warning(self.window, _("You cannot move torrent to a different partition. Please check your preferences. Or perhaps you are trying to move torrent's files to the same directory they are already stored?"))
            self.core.unique_IDs[event['unique_ID']].save_dir = event['message']
            self.core.pickle_state()

        elif event['event_type'] is self.core.constants['EVENT_FINISHED']:
            if event['message'] == "torrent has finished downloading":
                if self.config.get('enable_move_completed') and \
                    self.config.get('default_done_path') != \
                       self.core.get_pref('default_download_path') and \
                    self.core.unique_IDs[event['unique_ID']].save_dir != \
                       self.config.get('default_done_path'):
                    self.core.move_storage(event['unique_ID'],
                        self.config.get('default_done_path'))
                    # TODO Option to move torrent file as well? requires core mod?
        

    def update(self):
        from time import asctime
        import time
        if self.config.get('enable_auto_requests'):
            self.timer += 1
            if self.timer >= self.interval:
                if self.debug:
                    print asctime(), ": scanning requests dir", self.config.get("default_requests_path")
                import threading
                t = threading.Thread(target=self.pollrequests)
                t.start()
                self.timer = 0

    def pollrequests(self):
        import sys
        import os
        import shutil
        #
        # The interesting bit
        for elem in os.listdir(self.config.get("default_requests_path")):
            full_elem = os.path.join(self.config.get("default_requests_path"), elem)
            if self.config.get("enable_auto_moveto"):
                dst_full_elem = os.path.join(self.config.get("default_moveto_path"), elem)
                shutil.move(full_elem, dst_full_elem)
                full_elem = dst_full_elem
            if self.config.get("enable_auto_suffix"):
                # add suffix as requested
                full_elem_ext = full_elem + self.config.get("auto_suffix")
                os.rename(full_elem, full_elem_ext)
                full_elem = full_elem_ext
            # Interactive add torrent 
            self.interface.interactive_add_torrent(full_elem)
        sys.exit()
